@@ -17,7 +17,7 @@ use warnings;
 
 {
     package Doit;
-    our $VERSION = '0.024_51';
+    our $VERSION = '0.025_50';
 
     use constant IS_WIN => $^O eq 'MSWin32';
 }
@@ -179,7 +179,7 @@ use warnings;
 	if (defined $dir) {
 	    require Cwd;
 	    my $pwd = Cwd::getcwd();
-	    if (!defined $pwd) {
+	    if (!defined $pwd || $pwd eq '') { # XS variant returns undef, PP variant returns '' --- see https://rt.perl.org/Ticket/Display.html?id=132648
 		warning "No known current working directory";
 	    } else {
 		$scope_cleanup = new_scope_cleanup
@@ -1927,6 +1927,11 @@ use warnings;
 	$self->call_remote($method, @_); # XXX or use goto?
     }
 
+    sub _can_LANS {
+	require POSIX;
+	$^O eq 'linux' && (POSIX::uname())[2] !~ m{^([01]\.|2\.[01]\.)} # osvers >= 2.2, earlier versions did not have LANS
+    }
+
 }
 
 {
@@ -1971,16 +1976,14 @@ use warnings;
 
 	require File::Basename;
 	require IPC::Open2;
+	require POSIX;
 	require Symbol;
 
 	# Socket pathname, make it possible to find out
 	# old outdated sockets easily by including a
 	# timestamp. Also need to maintain a $socket_count,
 	# if the same script opens multiple sockets quickly.
-	my $sock_path = do {
-	    require POSIX;
-	    "/tmp/." . join(".", "doit", "sudo", POSIX::strftime("%Y%m%d_%H%M%S", gmtime), $<, $$, (++$socket_count)) . ".sock";
-	};
+	my $sock_path = "/tmp/." . join(".", "doit", "sudo", POSIX::strftime("%Y%m%d_%H%M%S", gmtime), $<, $$, (++$socket_count)) . ".sock";
 
 	# Make sure password has to be entered only once (if at all)
 	# Using 'sudo --validate' would be more correct, however,
@@ -2002,7 +2005,7 @@ use warnings;
 
 	# On linux use Linux Abstract Namespace Sockets ---
 	# invisible and automatically cleaned up. See man 7 unix.
-	my $LASN_PREFIX = $^O eq 'linux' ? '\0' : '';
+	my $LANS_PREFIX = $class->_can_LANS ? '\0' : '';
 
 	# Run the server
 	my @cmd_worker =
@@ -2011,8 +2014,8 @@ use warnings;
 	     Doit::_ScriptTools::self_require() .
 	     q{my $d = Doit->init; } .
 	     Doit::_ScriptTools::add_components(@components) .
-	     q{Doit::RPC::Server->new($d, "} . $LASN_PREFIX . $sock_path . q{", excl => 1, debug => } . ($debug?1:0) . q{)->run();} .
-	     ($LASN_PREFIX ? '' : q<END { unlink "> . $sock_path . q<" }>), # cleanup socket file, except if Linux Abstract Namespace Sockets are used
+	     q{Doit::RPC::Server->new($d, "} . $LANS_PREFIX . $sock_path . q{", excl => 1, debug => } . ($debug?1:0) . q{)->run();} .
+	     ($LANS_PREFIX ? '' : q<END { unlink "> . $sock_path . q<" }>), # cleanup socket file, except if Linux Abstract Namespace Sockets are used
 	     "--", ($dry_run? "--dry-run" : ())
 	    );
 	my $worker_pid = fork;
@@ -2028,7 +2031,7 @@ use warnings;
 	# access.
 	my($in, $out);
 	my @cmd_comm = ('sudo', @sudo_opts, $perl, "-I".File::Basename::dirname(__FILE__), "-MDoit", "-e",
-			q{Doit::Comm->comm_to_sock("} . $LASN_PREFIX . $sock_path . q{", debug => shift)}, !!$debug);
+			q{Doit::Comm->comm_to_sock("} . $LANS_PREFIX . $sock_path . q{", debug => shift)}, !!$debug);
 	warn "comm perl cmd: @cmd_comm\n" if $debug;
 	my $comm_pid = IPC::Open2::open2($out, $in, @cmd_comm);
 	$self->{rpc} = Doit::RPC::Client->new($out, $in, label => "sudo:", debug => $debug);
@@ -2060,6 +2063,8 @@ use warnings;
 	my $tty = delete $opts{tty};
 	my $port = delete $opts{port};
 	my $master_opts = delete $opts{master_opts};
+	my $dest_os = delete $opts{dest_os};
+	$dest_os = 'unix' if !defined $dest_os;
 	my $put_to_remote = delete $opts{put_to_remote} || 'rsync_put'; # XXX ideally this should be determined automatically
 	$put_to_remote =~ m{^(rsync_put|scp_put)$}
 	    or error "Valid values for put_to_remote: rsync_put or scp_put";
@@ -2080,7 +2085,12 @@ use warnings;
 	    and error "Connection error to $host: " . $ssh->error;
 	$self->{ssh} = $ssh;
 	{
-	    my $remote_cmd = "[ ! -d .doit/lib ] && mkdir -p .doit/lib";
+	    my $remote_cmd;
+	    if ($dest_os eq 'MSWin32') {
+		$remote_cmd = 'if not exist .doit\lib\ mkdir .doit\lib';
+	    } else {
+		$remote_cmd = "[ ! -d .doit/lib ] && mkdir -p .doit/lib";
+	    }
 	    if ($debug) {
 		info "Running '$remote_cmd' on remote";
 	    }
@@ -2093,23 +2103,43 @@ use warnings;
 	$ssh->$put_to_remote({verbose => $debug}, __FILE__, ".doit/lib/");
 	{
 	    my %seen_dir;
-	    for my $component (@components) {
+	    for my $component (
+			       @components,
+			       ( # add additional RPC components
+				$dest_os ne 'MSWin32' ? () :
+				do {
+				    (my $srcpath = __FILE__) =~ s{\.pm}{/WinRPC.pm};
+				    {relpath => "Doit/WinRPC.pm", path => $srcpath},
+				}
+			       )
+			      ) {
 		my $from = $component->{path};
 		my $to = $component->{relpath};
 		my $full_target = ".doit/lib/$to";
 		my $target_dir = File::Basename::dirname($full_target);
 		if (!$seen_dir{$target_dir}) {
-		    $ssh->system(\%ssh_run_opts, "[ ! -d $target_dir ] && mkdir -p $target_dir");
+		    my $remote_cmd;
+		    if ($dest_os eq 'MSWin32') {
+			(my $win_target_dir = $target_dir) =~ s{/}{\\}g;
+			$remote_cmd = "if not exist $win_target_dir mkdir $win_target_dir"; # XXX is this equivalent to mkdir -p?
+		    } else {
+			$remote_cmd = "[ ! -d $target_dir ] && mkdir -p $target_dir";
+		    }
+		    $ssh->system(\%ssh_run_opts, $remote_cmd);
 		    $seen_dir{$target_dir} = 1;
 		}
 		$ssh->$put_to_remote({verbose => $debug}, $from, $full_target);
 	    }
 	}
 
-	my $sock_path = do {
-	    require POSIX;
-	    "/tmp/." . join(".", "doit", "ssh", POSIX::strftime("%Y%m%d_%H%M%S", gmtime), $<, $$, int(rand(99999999))) . ".sock";
-	};
+	my $sock_path = (
+			 $dest_os eq 'MSWin32'
+			 ? join("-", "doit", "ssh", POSIX::strftime("%Y%m%d_%H%M%S", gmtime), int(rand(99999999)))
+			 : do {
+			     require POSIX;
+			     "/tmp/." . join(".", "doit", "ssh", POSIX::strftime("%Y%m%d_%H%M%S", gmtime), $<, $$, int(rand(99999999))) . ".sock";
+			 }
+			);
 
 	my @cmd;
 	if (defined $as) {
@@ -2120,7 +2150,23 @@ use warnings;
 	    }
 	} # XXX add ssh option -t? for password input?
 
-	my @cmd_worker =
+	my @cmd_worker;
+	if ($dest_os eq 'MSWin32') {
+	    @cmd_worker =
+	    (
+	     # @cmd not used here (no sudo)
+	     $perl, "-I.doit", "-I.doit\\lib", "-e",
+	     Doit::_ScriptTools::self_require($FindBin::RealScript) .
+	     q{use Doit::WinRPC; } .
+	     q{my $d = Doit->init; } .
+	     Doit::_ScriptTools::add_components(@components) .
+	     # XXX server cleanup? on signals? on END?
+	     q{Doit::WinRPC::Server->new($d, "} . $sock_path . q{", debug => } . ($debug?1:0).q{)->run();},
+	     "--", ($dry_run? "--dry-run" : ())
+	    );
+	    @cmd_worker = Doit::Win32Util::win32_quote_list(@cmd_worker);
+	} else {
+	    @cmd_worker =
 	    (
 	     @cmd, $perl, "-I.doit", "-I.doit/lib", "-e",
 	     Doit::_ScriptTools::self_require($FindBin::RealScript) .
@@ -2132,16 +2178,27 @@ use warnings;
 	     q{Doit::RPC::Server->new($d, "} . $sock_path . q{", excl => 1, debug => } . ($debug?1:0).q{)->run();},
 	     "--", ($dry_run? "--dry-run" : ())
 	    );
+	}
 	warn "remote perl cmd: @cmd_worker\n" if $debug;
 	my $worker_pid = $ssh->spawn(\%ssh_run_opts, @cmd_worker); # XXX what to do with worker pid?
 	$self->{worker_pid} = $worker_pid;
 
-	my @cmd_comm =
+	my @cmd_comm;
+	if ($dest_os eq 'MSWin32') {
+	    @cmd_comm =
+	    ($perl, "-I.doit\\lib", "-MDoit", "-MDoit::WinRPC", "-e",
+	     q{Doit::WinRPC::Comm->new("} . $sock_path . q{", debug => shift)->run},
+	     !!$debug,
+	    );
+	    @cmd_comm = Doit::Win32Util::win32_quote_list(@cmd_comm);
+	} else {
+	    @cmd_comm =
 	    (
 	     @cmd, $perl, "-I.doit/lib", "-MDoit", "-e",
 	     q{Doit::Comm->comm_to_sock("} . $sock_path . q{", debug => shift);},
 	     !!$debug,
 	    );
+	}
 	warn "comm perl cmd: @cmd_comm\n" if $debug;
 	my($out, $in, $comm_pid) = $ssh->open2(@cmd_comm);
 	$self->{comm_pid} = $comm_pid;
